#include <iostream>
#include <sstream>
#include <fstream>
#include <fstream>
#include <string>
#include <vector>
#include <cmath>
#include <limits>
#include <iomanip>

#include "appl_grid/appl_grid.h"
#include "appl_grid/appl_timer.h"

#include "TH1D.h"
#include "TPad.h"
#include "TROOT.h"
#include "TObjArray.h"
#include "TFile.h"
#include "TCanvas.h"
#include "TStyle.h"
#include "TDirectory.h"
#include "TTree.h"
#include "TGaxis.h"
#include "TLegend.h"

//#include "myown.hpp"
// lhapdf routines
#include "LHAPDF/LHAPDF.h"
extern "C" void evolvepdf_(const double& , const double& , double* ); 
extern "C" double alphaspdf_(const double& Q);


using namespace std;

int main(int argc, char** argv){
  string gridname, _pdfname="CT14nnlo", logxy;
  int iset=0;
  if(argc==1) {
		cout<<"stand_PDFset grid PDF iset logxy"<<endl;
		return 0;
	}
string instr;
for(int iarg=1;iarg<argc;iarg++){
	instr=argv[iarg];
	if(instr.find("root")!=string::npos) gridname=instr;
  else if(instr.find("log")!=string::npos) logxy=instr;
  else if(instr.find_first_not_of("-.0123456789") == string::npos) iset=atoi(argv[iarg]);
  else _pdfname = instr;
}
// use a default atlas inclusive grid
//***********************************************
  cout<<"Usage method"<<endl;
  cout<<"stand_PDFunc "<< gridname<<" "<<_pdfname<<" "<<iset<<endl;
//***********************************************

   TCanvas *c=new TCanvas("c","canvas",600,800);
   gStyle->SetPadTickX(1);
   gStyle->SetPadTickY(1);
   gStyle->SetOptTitle(0);
   gStyle->SetOptStat(0);

	 c->cd();
   TPad *pad1 = new TPad("pad1", "pad1", 0, 0.4, 1, 1.0);
   pad1->SetBottomMargin(0); // Upper and lower plot are joined
 	 if(logxy=="logx"||logxy=="logxy") pad1->SetLogx();
	 if(logxy=="logy"||logxy=="logxy") pad1->SetLogy();
   pad1->Draw();
   pad1->cd();

  // Go back to the main canvas before defining pad2
   c->cd();
   TPad *pad2 = new TPad("pad2", "pad2", 0, 0.05, 1, 0.4);
   pad2->SetTopMargin(0);
   pad2->SetBottomMargin(0.1);
   //pad2->SetGridy(); // vertical grid
   if(logxy=="logx"||logxy=="logxy") pad2->SetLogx();
   pad2->Draw();
   pad2->cd();       // pad2 becomes the current pad

	 /*
   static const int Nl=100, Nc=3;
   double X[Nl][Nc];
   int il=0, ic=0;
   string line;
   fstream in(KFtxt.c_str());
   while(getline(in, line))
   {
     cout<<line<<endl;
     stringstream ss(line);
     for(ic=0;ic<3;ic++)  ss>>X[il][ic];
     il++;
   }
   cout<<il<<endl;
*/
  std::cout << "reading grid " << gridname << std::endl; 
  TFile *f = TFile::Open(gridname.c_str());
//  f->cd();
  TDirectory* grid=f->GetDirectory("grid");
  TH1D* histo;
  grid->GetObject("reference",histo);
  pad1->cd();
  histo->SetLineColor(kBlack);
  histo->SetLineWidth(2);
//  histo->Draw("histo");
	if(gridname.find("mcfm")!=string::npos) histo->Scale(1e-3);
  TH1D* hbench=(TH1D*)histo->Clone("hbench");
  for(int ibin=1; ibin<histo->GetSize()-1; ibin++)  hbench->SetBinError(ibin, 0);

  // get name of grid from user and create from grid file
  appl::grid g(gridname);
  g.trim(); // trim away uneeded memory
  /// print the grid documentation 
  std::cout << g.getDocumentation() << std::endl; 

  // initialise lhapdf
  const LHAPDF::PDFSet set(_pdfname);
  const size_t Nset = set.size();
  cout << "LHAPDF "<<_pdfname<< " 1+"<<Nset-1<<" error sets" <<endl;

  size_t lastindex = gridname.find_last_of(".");
  string outname = gridname.substr(0, lastindex);
  outname+="_"+_pdfname+"_"+to_string(iset);
  cout<<outname<<endl;
  TFile* fPDF = new TFile((outname+".root").c_str(), "recreate");
  fPDF->cd();
  TObjArray *histos = new TObjArray(0);

  ofstream outfile((outname+".txt").c_str());
//  outfile<<left<<"* " <<gridname<<" "<<_pdfname<<endl;
//  outfile<<"* BinLo   BinHi   Ref      Conv     Ratio    KF       NNLO=Conv*KF" <<endl;
//	outfile<<"Theory Column "<<endl;
//  outfile<<" 7 "<<endl;

TH1D* hxsec[Nset];
//for(int iset=0;iset<Nset;iset++)
//{
  LHAPDF::initPDFSet( _pdfname, iset );
  // initpdfset_(_pdfname.c_str());
  // initpdf_( iset );

  // do the convolution into a vector
  std::cout << "doing standalone convolution" << std::endl; 

  //  struct timeval atimer = appl_timer_start(); 
  //std::vector<double>  xsec = g.vconvolute( evolvepdf_, alphaspdf_ ); 
  //  double atime = appl_timer_stop(atimer); 
  //  std::cout << "time " << atime << " ms" << std::endl;
  //for ( int i=0 ; i<xsec.size() ; i++ ) std::cout << "xsec[" << i << "]\t= " << xsec[i] << std::endl;  

  // do the convolution into a TH1D
  //  atimer = appl_timer_start(); 
  hxsec[iset] = g.convolute( evolvepdf_, alphaspdf_, 1 ); 
	if(gridname.find("mcfm")!=string::npos)  hxsec[iset]->Scale(1e-3);

//  outfile<<""<<_pdfname<<" : set "<< iset<<endl;
  for(int i=1;i<hxsec[iset]->GetSize()-1;i++)
  {  std::cout<<setw(2)<<i<<" "<<scientific<<setprecision(3)<<setw(10)
		<<hxsec[iset]->GetBinLowEdge(i)<<"--"<<hxsec[iset]->GetBinLowEdge(i+1)
  	<<" ref "<<histo->GetBinContent(i)<<" conv "<<hxsec[iset]->GetBinContent(i)
  	<<" ratio "<<hxsec[iset]->GetBinContent(i)/histo->GetBinContent(i)<<std::endl;
     outfile<<"  "<<setw(7)<<hxsec[iset]->GetBinLowEdge(i)<<" "<<setw(7)<<hxsec[iset]->GetBinLowEdge(i+1)
  	<<" "<<setw(8)<<histo->GetBinContent(i)<<" "<<setw(8)<<hxsec[iset]->GetBinContent(i)
  	<<" "<<setw(8)<<hxsec[iset]->GetBinContent(i)/histo->GetBinContent(i)<<endl;
//  <<" "<<setw(8)<<X[i-1][2]<<" "<<setw(8)<<hxsec[iset]->GetBinContent(i)*X[i-1][2]<<std::endl;
  }
	cout<<endl;
//}
/*
for(int ibin=1;ibin<hbench->GetSize()-1;ibin++)
{
  double unc=0;
  for(int iset=1;iset<Nset; iset=iset+2)
  {  
    double Errp=hxsec[iset]->GetBinContent(ibin);
    double Errm=hxsec[iset+1]->GetBinContent(ibin);
    unc+=pow(Errp-Errm,2.0);
  }
  unc=0.5*sqrt(unc);
  hxsec[0]->SetBinError(ibin,unc);  
}
*/
//  hxsec[0]->Print("all");
//  hPDFunc(hxsec, Nset-1);

  fPDF->cd();
  histos->Add(hxsec[iset]);


  TGaxis::SetMaxDigits(3);
  pad1->cd();
  double Ymax=max(hxsec[iset]->GetMaximum(),histo->GetMaximum());
  double Ymin=min(hxsec[iset]->GetMinimum(),histo->GetMinimum());
  hxsec[iset]->GetYaxis()->SetRangeUser(Ymin*0.91,Ymax*1.09);
  hxsec[iset]->SetLineColor(kRed);
  hxsec[iset]->SetLineWidth(2);
	hxsec[iset]->SetLineStyle(2);
  hxsec[iset]->Draw("hist same");
//  hxsec[iset]->Print("all");
  histo->SetLineColor(kBlue);
  histo->SetLineWidth(2);  
	histo->SetLineStyle(1);
  histo->Draw("hist same");
//  histo->Print("all");

  TLegend* leg=new TLegend(0.15,0.01,0.50,0.30);
//  SetLegend(leg);
  leg->SetFillStyle(0);
  leg->SetBorderSize(0);
  leg->SetTextSize(0.06);
  leg->SetTextFont(132);
  leg->SetHeader((gridname.substr(0,gridname.find_last_of("."))).c_str());
  leg->AddEntry(histo,"MCFM Ref.","l");
  leg->AddEntry(hxsec[iset],(_pdfname+" set "+to_string(iset)).c_str(),"l");
  leg->Draw();

	pad2->cd();
  TH1D* hRatio=(TH1D*)hxsec[iset]->Clone("hRatio");
  hRatio->Divide(hbench);
  TH1D* hRef=(TH1D*)histo->Clone("hRef");
  hRef->Divide(hbench);
  double Rmax=1, Rmin=1.;
  for(int ibin=1;ibin<hbench->GetSize()-1;ibin++)
  {  Rmax=max(Rmax,hRatio->GetBinContent(ibin));
     Rmin=min(Rmin,hRatio->GetBinContent(ibin));
     Rmax=max(Rmax,hRef->GetBinContent(ibin));
     Rmin=min(Rmin,hRef->GetBinContent(ibin));
  }
  cout<<"Ratio range "<<Rmin<<" "<<Rmax<<endl;
	double eps=Rmax-Rmin;
  hRatio->GetYaxis()->SetRangeUser(Rmin*(1-0.5*eps),Rmax*(1+0.5*eps));

  hRatio->GetXaxis()->SetLabelSize(0.05);
  hRatio->GetYaxis()->SetLabelSize(0.05);
	if(logxy=="logx"||logxy=="logxy"){
		hRatio->GetXaxis()->SetMoreLogLabels();
 		hRatio->GetXaxis()->SetNoExponent();
	}
	hRatio->Draw("hist");
	hRef->Draw("hist same");
//  hRef->Print("all");
//  hRatio->Print("all");

  c->Print((outname+".pdf").c_str());

  fPDF->cd();
  histos->Write();
  fPDF->Write();
  fPDF->Close();
  return 0;
}
